package com.fdkankan.scene.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.fdkankan.scene.entity.Folder;
import com.fdkankan.scene.vo.FolderVO;
import java.util.List;
import org.apache.ibatis.annotations.Mapper;

/**
 * <p>
 * 文件夹表 Mapper 接口
 * </p>
 *
 * @author dengsixing
 * @since 2021-12-24
 */
@Mapper
public interface ISceneUpgradeMapper extends BaseMapper {

    public void transferScenePro(Long sceneProId);

    public void transferSceneProExt(Long sceneProId);

    public void transferScenePlus(Long sceneProId);

    public void transferScenePlusExt(Long sceneProId);

    public void transferSceneProEdit(Long sceneProEditId);

    public void transferSceneProEditExt(Long sceneProEditId);

    public void deleteSceneProEdit(Long sceneProEditId);

    public void deleteSceneProEditExt(Long sceneProEditId);

    public void deleteScenePlus(Long sceneProId);

    public void deleteScenePlusExt(Long sceneProId);

    public void deleteScenePro(Long sceneProId);

    public void deleteSceneProExt(Long sceneProId);


}
